// *******************************************************************
// File    : ColorProgressBar.cs 
// Author  : Igor Tolmachev
// Email   : info@itsamples.com
// Web     : www.itsamples.com 
// You are free to use/modify this code but leave this header intact.
// *******************************************************************

using System;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Windows.Forms;
using System.Drawing;
using System.Drawing.Drawing2D;

namespace ExifDataChanger
{
	[Description("Color Progress Bar")]
	[ToolboxBitmap(typeof(ProgressBar))]
	public class ColorProgressBar : System.Windows.Forms.Control
	{	
	
		//
		// set default values
		//
		private int m_nValue = 0;
		private int m_nMinimum = 0;
		private int m_nMaximum = 100;
		private int m_nStep = 10;

		private FillStyles m_FillStyle = FillStyles.Solid;

		private Color m_crBarColor = Color.FromArgb(0, 164, 0);
		private Color m_crBorderColor = Color.Black;
		private Pen  m_penPaneLine = new Pen(SystemColors.ControlDark, 1);

		public enum FillStyles
		{
			Solid,
			Dashed
		}

		public ColorProgressBar()
		{
			base.Size = new Size(150, 15);
			SetStyle(
				ControlStyles.AllPaintingInWmPaint |
				ControlStyles.ResizeRedraw |
				ControlStyles.DoubleBuffer,
				true
				);
		}

		[Description( "ColorProgressBar color")]
		[Category( "ColorProgressBar" )]
		public Color BarColor
		{
			get
			{
				return m_crBarColor;
			}
			set
			{
				m_crBarColor = value;
				this.Invalidate();
			}
		}

		[Description( "ColorProgressBar fill style")]
		[Category( "ColorProgressBar" )]
		public FillStyles FillStyle
		{
			get
			{
				return m_FillStyle;
			}
			set
			{
				m_FillStyle = value;
				this.Invalidate();
			}
		}

		[Description( "The current value for the ColorProgressBar, "+
			 "in the range specified by the Minimum and Maximum properties." )]
		[Category( "ColorProgressBar" )]
		// the rest of the Properties windows must be updated when this peroperty is changed.
		[RefreshProperties(RefreshProperties.All)]
		public int Value
		{
			get
			{
				return m_nValue;
			}
			set
			{
				if (value < m_nMinimum)
				{
					throw new ArgumentException("'"+value+"' is not a valid value for 'Value'.\n"+
						"'Value' must be between 'Minimum' and 'Maximum'.");
				}

				if (value > m_nMaximum)
				{
					throw new ArgumentException("'"+value+"' is not a valid value for 'Value'.\n"+
						"'Value' must be between 'Minimum' and 'Maximum'.");
				}

				m_nValue = value;			
				this.Invalidate();
			}
		}
		
		[Description("The lower bound of the range this ColorProgressbar is working with.")]
		[Category("ColorProgressBar")]
		[RefreshProperties(RefreshProperties.All)]
		public int Minimum
		{
			get
			{
				return m_nMinimum;
			}
			set
			{
				m_nMinimum = value;

				if (m_nMinimum > m_nMaximum)
					m_nMaximum = m_nMinimum;
				if (m_nMinimum > m_nValue)
					m_nValue = m_nMinimum;

				this.Invalidate();
			}
		}

		[Description("The uppper bound of the range this ColorProgressbar is working with.")]
		[Category("ColorProgressBar")]
		[RefreshProperties(RefreshProperties.All)]
		public int Maximum
		{
			get
			{
				return m_nMaximum;
			}
			set
			{
				m_nMaximum = value;

				if (m_nMaximum < m_nValue)
					m_nValue = m_nMaximum;
				if (m_nMaximum < m_nMinimum)
					m_nMinimum = m_nMaximum;

				this.Invalidate();
			}
		}

		[Description("The amount to jump the current value of the control by when the Step() method is called.")]
		[Category("ColorProgressBar")]		
		public int Step
		{
			get
			{
				return m_nStep;
			}
			set
			{
				m_nStep = value;
				this.Invalidate();
			}
		}

		[Description("The border color of ColorProgressBar")]
		[Category("ColorProgressBar")]		
		public Color BorderColor
		{
			get
			{
				return m_crBorderColor;
			}
			set
			{
				m_crBorderColor = value;
				this.Invalidate();
			}
		}
		
		//
		// Call the PerformStep() method to increase the value displayed by the amount set in the Step property
		//
		public void PerformStep()
		{
			if (m_nValue < m_nMaximum)
				m_nValue += m_nStep;
			else
				m_nValue = m_nMaximum;

			this.Invalidate();
		}
		
		//
		// Call the PerformStepBack() method to decrease the value displayed by the amount set in the Step property
		//
		public void PerformStepBack()
		{
			if (m_nValue > m_nMinimum)
				m_nValue -= m_nStep;
			else
				m_nValue = m_nMinimum;

			this.Invalidate();
		}

		//
		// Call the Increment() method to increase the value displayed by an integer you specify
		// 
		public void Increment(int value)
		{
			if (m_nValue < m_nMaximum)
				m_nValue += value;
			else
				m_nValue = m_nMaximum;

			this.Invalidate();
		}
		
		//
		// Call the Decrement() method to decrease the value displayed by an integer you specify
		// 
		public void Decrement(int value)
		{
			if (m_nValue > m_nMinimum)
				m_nValue -= value;
			else
				m_nValue = m_nMinimum;

			this.Invalidate();
		}
		
		protected override void OnPaint(System.Windows.Forms.PaintEventArgs e)
		{
			//
			// Fill background
			//
			SolidBrush brBackBrush = new SolidBrush(SystemColors.Control);

			e.Graphics.FillRectangle(brBackBrush, 1,1,this.Width-1,this.Height-1);
			brBackBrush.Dispose();
			
			// 
			// Check for value
			//
			if (m_nMaximum == m_nMinimum || m_nValue == 0)
			{
				// Draw border only and exit;
				DrawBorder(e.Graphics);
				return;
			}

			//
			// The following is the width of the bar. This will vary with each value.
			//
			int nFillWidth = (this.Width * m_nValue) / (m_nMaximum - m_nMinimum);
			
			//
			// GDI+ doesn't like rectangles 0px wide or high
			//
			if (nFillWidth == 0)
			{
				// Draw border only and exit;
				DrawBorder(e.Graphics);
				return;
			}

			//Make the bars of the progress complete, just like XP bars
			if(nFillWidth%8 != 0)
				nFillWidth += (8-nFillWidth % 8);

			//
			// Rectangles for upper and lower half of bar
			//
			Rectangle rcBar = new Rectangle(3,2, nFillWidth, this.Height-5);
			//Rectangle topRect = new Rectangle(0, 0, fillWidth, this.Height / 2);
			//Rectangle buttomRect = new Rectangle(0, this.Height / 2, fillWidth, this.Height / 2);

			//
			// The gradient brush
			//
			LinearGradientBrush brGradient;			

			//
			// Paint upper half
			//
			brGradient = new LinearGradientBrush(rcBar, SystemColors.Control, m_crBarColor, 90.0f);
			float[] nIntensities = {0.1f, 1.0f, 1.0f, 1.0f, 1.0f, 0.85f, 0.1f}; 
			float[] nPositions =   {0.0f, 0.2f, 0.5f, 0.5f, 0.5f, 0.8f, 1.0f};

			// create a Blend object and assign it
			Blend blend = new Blend();
			blend.Factors = nIntensities;
			blend.Positions = nPositions; 
			brGradient.Blend = blend;

			e.Graphics.FillRectangle(brGradient, rcBar);
			brGradient.Dispose();

			//
			// Calculate separator's setting
			//
			// Separator is made smaller like the XP progress
			int nSepWidth = (int)(this.Height * .67);
			int nSepCount = (int)(nFillWidth / nSepWidth);

			//
			// Paint separators
			//
			switch (m_FillStyle)
			{
				case FillStyles.Dashed:
					// Draw each separator line
					for (int i = 0; i <= nSepCount; i++)
					{
						e.Graphics.DrawLine(new Pen(SystemColors.Control, 2),
							nSepWidth * i+3, 0, nSepWidth * i+3, this.Height);
					}
					break;

				case FillStyles.Solid:
				default:
					break;
			}

			//
			// Draw border and exit
			//
			DrawBorder(e.Graphics);
		}

		//
		// Draw border
		//
		protected void DrawBorder(Graphics g)
		{
			Rectangle borderRect = new Rectangle(0, 1, ClientRectangle.Width-1, ClientRectangle.Height-3);
			g.DrawRectangle(m_penPaneLine, borderRect);
		}
	
		protected override void Dispose( bool disposing )
		{
			if(disposing)
			{
				m_penPaneLine.Dispose();
			}

			base.Dispose( disposing );
		}

	}
}