// *******************************************************************
// File    : PhotoHelper.cs 
// Author  : Igor Tolmachev
// Email   : info@itsamples.com
// Web     : www.itsamples.com 
// You are free to use/modify this code but leave this header intact.
// *******************************************************************

using System;
using System.Drawing;
using System.Drawing.Imaging;

namespace ExifDataChanger
{
	/// <summary>
	/// Summary description for PhotoHelper.
	/// </summary>
	public class PhotoHelper
	{
		public PhotoHelper()
		{
			//
			// TODO: Add constructor logic here
			//
		}

		/// <summary>
		/// Defines the DOUBLE format used when formatting Rational and 
		/// SRational Property Tag types.</summary>
		private const string DOUBLETYPE_FORMAT = "0.0####";

		/// <summary>Index byte jump for a PropertyItem's byte array of Shorts.</summary>
		private const int BYTEJUMP_SHORT		= 2;
		/// <summary>Index byte jump for a PropertyItem's byte array of Longs.</summary>
		private const int BYTEJUMP_LONG			= 4;
		/// <summary>Index byte jump for a PropertyItem's byte array of Rationals.</summary>
		private const int BYTEJUMP_RATIONAL		= 8;
		/// <summary>Index byte jump for a PropertyItem's byte array of SLongs.</summary>
		private const int BYTEJUMP_SLONG		= 4;
		/// <summary>Index byte jump for a PropertyItem's byte array of SRationals.</summary>
		private const int BYTEJUMP_SRATIONAL	= 8;

		// //////////////////////////////////////////////////////////
		// Image Property Tag Types 
		// //////////////////////////////////////////////////////////
		/// <summary>BYTE</summary>
		/// <remarks>An 8-bit unsigned integer.</remarks>
		private const int PropertyTagTypeByte        = 1;
		/// <summary>ASCII</summary>
		/// <remarks>An 8-bit byte containing one 7-bit ASCII code. 
		///          The final byte is terminated with NULL.</remarks>
		private const int PropertyTagTypeASCII       = 2;
		/// <summary>SHORT</summary>
		/// <remarks>A 16-bit (2-byte) unsigned integer</remarks>
		private const int PropertyTagTypeShort       = 3;
		/// <summary>LONG</summary>
		/// <remarks>A 32-bit (4-byte) unsigned integer</remarks>
		private const int PropertyTagTypeLong        = 4;
		/// <summary>RATIONAL</summary>
		/// <remarks>Two LONGs. 
		///          The first LONG is the numerator.
		///          The second LONG is the denominator.</remarks>
		private const int PropertyTagTypeRational    = 5;
		/// <summary>UNDEFINED</summary>
		/// <remarks>An 8-bit byte that can take any value 
		///          depending on the field definition</remarks>
		private const int PropertyTagTypeUndefined   = 7;
		/// <summary>SLONG</summary>
		/// <remarks>A 32-bit (4-byte) signed integer (2's complement notation)</remarks>
		private const int PropertyTagTypeSLong       = 9;
		/// <summary>SRATIONAL</summary>
		/// <remarks>Two SLONGs.
		///          The first SLONG is the numerator.
		///          The second SLONG is the denominator.</remarks>
		private const int PropertyTagTypeSRational   = 10;

		public static string GetPropertyDesriptionFromName(string strName)
		{
			if(strName.Equals("ImageWidth")) return GetPropertyDescriptionFromId(256);
			else if(strName.Equals("ImageHeight")) return GetPropertyDescriptionFromId(257);
			else if(strName.Equals("BitsPerSample")) return GetPropertyDescriptionFromId(258);
			else if(strName.Equals("Compression")) return GetPropertyDescriptionFromId(259);
			else if(strName.Equals("PhotometricInterp")) return GetPropertyDescriptionFromId(262);
			else if(strName.Equals("ThreshHolding")) return GetPropertyDescriptionFromId(263);
			else if(strName.Equals("CellWidth")) return GetPropertyDescriptionFromId(264);
			else if(strName.Equals("CellHeight")) return GetPropertyDescriptionFromId(265);
			else if(strName.Equals("FillOrder")) return GetPropertyDescriptionFromId(266);
			else if(strName.Equals("DocumentName")) return GetPropertyDescriptionFromId(269);
			else if(strName.Equals("ImageDescription")) return GetPropertyDescriptionFromId(270);
			else if(strName.Equals("EquipMake")) return GetPropertyDescriptionFromId(271);
			else if(strName.Equals("EquipModel")) return GetPropertyDescriptionFromId(272);
			else if(strName.Equals("StripOffsets")) return GetPropertyDescriptionFromId(273);
			else if(strName.Equals("Orientation")) return GetPropertyDescriptionFromId(274);
			else if(strName.Equals("SamplesPerPixel")) return GetPropertyDescriptionFromId(277);
			else if(strName.Equals("RowsPerStrip")) return GetPropertyDescriptionFromId(278);
			else if(strName.Equals("StripBytesCount")) return GetPropertyDescriptionFromId(279);
			else if(strName.Equals("MinSampleValue")) return GetPropertyDescriptionFromId(280);
			else if(strName.Equals("MaxSampleValue")) return GetPropertyDescriptionFromId(281);
			else if(strName.Equals("XResolution")) return GetPropertyDescriptionFromId(282);
			else if(strName.Equals("YResolution")) return GetPropertyDescriptionFromId(283);
			else if(strName.Equals("PlanarConfig")) return GetPropertyDescriptionFromId(284);
			else if(strName.Equals("PageName")) return GetPropertyDescriptionFromId(285);
			else if(strName.Equals("XPosition")) return GetPropertyDescriptionFromId(286);
			else if(strName.Equals("YPosition")) return GetPropertyDescriptionFromId(287);
			else if(strName.Equals("FreeOffset")) return GetPropertyDescriptionFromId(288);
			else if(strName.Equals("FreeByteCounts")) return GetPropertyDescriptionFromId(289);
			else if(strName.Equals("GrayResponseUnit")) return GetPropertyDescriptionFromId(290);
			else if(strName.Equals("GrayResponseCurve")) return GetPropertyDescriptionFromId(291);
			else if(strName.Equals("T4Option")) return GetPropertyDescriptionFromId(292);
			else if(strName.Equals("T6Option")) return GetPropertyDescriptionFromId(293);
			else if(strName.Equals("ResolutionUnit")) return GetPropertyDescriptionFromId(296);
			else if(strName.Equals("PageNumber")) return GetPropertyDescriptionFromId(297);
			else if(strName.Equals("TransferFunction")) return GetPropertyDescriptionFromId(301);
			else if(strName.Equals("SoftwareUsed")) return GetPropertyDescriptionFromId(305);
			else if(strName.Equals("DateTime")) return GetPropertyDescriptionFromId(306);
			else if(strName.Equals("Artist")) return GetPropertyDescriptionFromId(315);
			else if(strName.Equals("HostComputer")) return GetPropertyDescriptionFromId(316);
			else if(strName.Equals("Predictor")) return GetPropertyDescriptionFromId(317);
			else if(strName.Equals("WhitePoint")) return GetPropertyDescriptionFromId(318);
			else if(strName.Equals("PrimaryChromaticities")) return GetPropertyDescriptionFromId(319);
			else if(strName.Equals("ColorMap")) return GetPropertyDescriptionFromId(320);
			else if(strName.Equals("HalftoneHints")) return GetPropertyDescriptionFromId(321);
			else if(strName.Equals("TileWidth")) return GetPropertyDescriptionFromId(322);
			else if(strName.Equals("TileLength")) return GetPropertyDescriptionFromId(323);
			else if(strName.Equals("TileOffset")) return GetPropertyDescriptionFromId(324);
			else if(strName.Equals("TileByteCounts")) return GetPropertyDescriptionFromId(325);
			else if(strName.Equals("InkSet")) return GetPropertyDescriptionFromId(332);
			else if(strName.Equals("InkNames")) return GetPropertyDescriptionFromId(333);
			else if(strName.Equals("NumberOfInks")) return GetPropertyDescriptionFromId(334);
			else if(strName.Equals("DotRange")) return GetPropertyDescriptionFromId(336);
			else if(strName.Equals("TargetPrinter")) return GetPropertyDescriptionFromId(337);
			else if(strName.Equals("ExtraSamples")) return GetPropertyDescriptionFromId(338);
			else if(strName.Equals("SampleFormat")) return GetPropertyDescriptionFromId(339);
			else if(strName.Equals("SMinSampleValue")) return GetPropertyDescriptionFromId(340);
			else if(strName.Equals("SMaxSampleValue")) return GetPropertyDescriptionFromId(341);
			else if(strName.Equals("TransferRange")) return GetPropertyDescriptionFromId(342);
			else if(strName.Equals("Proc")) return GetPropertyDescriptionFromId(512);
			else if(strName.Equals("InterFormat")) return GetPropertyDescriptionFromId(513);
			else if(strName.Equals("InterLength")) return GetPropertyDescriptionFromId(514);
			else if(strName.Equals("RestartInterval")) return GetPropertyDescriptionFromId(515);
			else if(strName.Equals("LosslessPredictors")) return GetPropertyDescriptionFromId(517);
			else if(strName.Equals("PointTransforms")) return GetPropertyDescriptionFromId(518);
			else if(strName.Equals("QTables")) return GetPropertyDescriptionFromId(519);
			else if(strName.Equals("DCTables")) return GetPropertyDescriptionFromId(520);
			else if(strName.Equals("ACTables")) return GetPropertyDescriptionFromId(521);
			else if(strName.Equals("YCbCrCoefficients")) return GetPropertyDescriptionFromId(529);
			else if(strName.Equals("YCbCrSubsampling")) return GetPropertyDescriptionFromId(530);
			else if(strName.Equals("YCbCrPositioning")) return GetPropertyDescriptionFromId(531);
			else if(strName.Equals("REFBlackWhite")) return GetPropertyDescriptionFromId(532);
			else if(strName.Equals("Gamma")) return GetPropertyDescriptionFromId(769);
			else if(strName.Equals("ICCProfileDescriptor")) return GetPropertyDescriptionFromId(770);
			else if(strName.Equals("ImageTitle")) return GetPropertyDescriptionFromId(800);
			else if(strName.Equals("ResolutionXUnit")) return GetPropertyDescriptionFromId(20481);
			else if(strName.Equals("ResolutionYUnit")) return GetPropertyDescriptionFromId(20482);
			else if(strName.Equals("ResolutionXLengthUnit")) return GetPropertyDescriptionFromId(20483);
			else if(strName.Equals("ResolutionYLengthUnit")) return GetPropertyDescriptionFromId(20484);
			else if(strName.Equals("JPEGQuality")) return GetPropertyDescriptionFromId(20496);
			else if(strName.Equals("Copyright")) return GetPropertyDescriptionFromId(33432);
			else if(strName.Equals("ExposureTime")) return GetPropertyDescriptionFromId(33434);
			else if(strName.Equals("FNumber")) return GetPropertyDescriptionFromId(33437);
			else if(strName.Equals("IFD")) return GetPropertyDescriptionFromId(34665);
			else if(strName.Equals("ICCProfile")) return GetPropertyDescriptionFromId(34675);
			else if(strName.Equals("ExposureProg")) return GetPropertyDescriptionFromId(34850);
			else if(strName.Equals("SpectralSense")) return GetPropertyDescriptionFromId(34852);
			else if(strName.Equals("GpsIFD")) return GetPropertyDescriptionFromId(34853);
			else if(strName.Equals("ISOSpeed")) return GetPropertyDescriptionFromId(34855);
			else if(strName.Equals("OECF")) return GetPropertyDescriptionFromId(34856);
			else if(strName.Equals("ExifVersion")) return GetPropertyDescriptionFromId(36864);
			else if(strName.Equals("DateTimeOriginal")) return GetPropertyDescriptionFromId(36867);
			else if(strName.Equals("DateTimeDigitized")) return GetPropertyDescriptionFromId(36868);
			else if(strName.Equals("ComponentsConfiguration")) return GetPropertyDescriptionFromId(37121);
			else if(strName.Equals("CompBPP")) return GetPropertyDescriptionFromId(37122);
			else if(strName.Equals("ShutterSpeed")) return GetPropertyDescriptionFromId(37377);
			else if(strName.Equals("Aperture")) return GetPropertyDescriptionFromId(37378);
			else if(strName.Equals("Brightness")) return GetPropertyDescriptionFromId(37379);
			else if(strName.Equals("ExposureBias")) return GetPropertyDescriptionFromId(37380);
			else if(strName.Equals("MaxAperture")) return GetPropertyDescriptionFromId(37381);
			else if(strName.Equals("SubjectDist")) return GetPropertyDescriptionFromId(37382);
			else if(strName.Equals("MeteringMode")) return GetPropertyDescriptionFromId(37383);
			else if(strName.Equals("LightSource")) return GetPropertyDescriptionFromId(37384);
			else if(strName.Equals("Flash")) return GetPropertyDescriptionFromId(37385);
			else if(strName.Equals("FocalLength")) return GetPropertyDescriptionFromId(37386);
			else if(strName.Equals("SubjectArea")) return GetPropertyDescriptionFromId(37396);
			else if(strName.Equals("MakerNote")) return GetPropertyDescriptionFromId(37500);
			else if(strName.Equals("UserComment")) return GetPropertyDescriptionFromId(37510);
			else if(strName.Equals("DTSubsec")) return GetPropertyDescriptionFromId(37520);
			else if(strName.Equals("DTOrigSS")) return GetPropertyDescriptionFromId(37521);
			else if(strName.Equals("DTDigSS")) return GetPropertyDescriptionFromId(37522);
			else if(strName.Equals("FlashpixVersion")) return GetPropertyDescriptionFromId(40960);
			else if(strName.Equals("ColorSpace")) return GetPropertyDescriptionFromId(40961);
			else if(strName.Equals("PixXDimension")) return GetPropertyDescriptionFromId(40962);
			else if(strName.Equals("PixYDimension")) return GetPropertyDescriptionFromId(40963);
			else if(strName.Equals("RelatedWav")) return GetPropertyDescriptionFromId(40964);
			else if(strName.Equals("Interop")) return GetPropertyDescriptionFromId(40965);
			else if(strName.Equals("FlashEnergy")) return GetPropertyDescriptionFromId(41483);
			else if(strName.Equals("SpatialFR")) return GetPropertyDescriptionFromId(41484);
			else if(strName.Equals("FocalXRes")) return GetPropertyDescriptionFromId(41486);
			else if(strName.Equals("FocalYRes")) return GetPropertyDescriptionFromId(41487);
			else if(strName.Equals("FocalResUnit")) return GetPropertyDescriptionFromId(41488);
			else if(strName.Equals("SubjectLoc")) return GetPropertyDescriptionFromId(41492);
			else if(strName.Equals("ExposureIndex")) return GetPropertyDescriptionFromId(41493);
			else if(strName.Equals("SensingMethod")) return GetPropertyDescriptionFromId(41495);
			else if(strName.Equals("FileSource")) return GetPropertyDescriptionFromId(41728);
			else if(strName.Equals("SceneType")) return GetPropertyDescriptionFromId(41729);
			else if(strName.Equals("CfaPattern")) return GetPropertyDescriptionFromId(41730);
			else if(strName.Equals("CustomRendered")) return GetPropertyDescriptionFromId(41985);
			else if(strName.Equals("ExposureMode")) return GetPropertyDescriptionFromId(41986);
			else if(strName.Equals("WhiteBalance")) return GetPropertyDescriptionFromId(41987);
			else if(strName.Equals("DigitalZoomRatio")) return GetPropertyDescriptionFromId(41988);
			else if(strName.Equals("FocalLengthIn35mmFilm")) return GetPropertyDescriptionFromId(41989);
			else if(strName.Equals("SceneCaptureType")) return GetPropertyDescriptionFromId(41990);
			else if(strName.Equals("GainControl")) return GetPropertyDescriptionFromId(41991);
			else if(strName.Equals("Contrast")) return GetPropertyDescriptionFromId(41992);
			else if(strName.Equals("Saturation")) return GetPropertyDescriptionFromId(41993);
			else if(strName.Equals("Sharpness")) return GetPropertyDescriptionFromId(41994);
			else if(strName.Equals("DeviceSettingDescription")) return GetPropertyDescriptionFromId(41995);
			else if(strName.Equals("SubjectDistanceRange")) return GetPropertyDescriptionFromId(41996);
			else if(strName.Equals("ImageUniqueID")) return GetPropertyDescriptionFromId(42016);

			return "";
		}
	
		public static string GetPropertyNameFromId(int nID)
		{
			switch(nID)
			{
				case 256:
					return "ImageWidth";
				case 257:
					return "ImageHeight";
				case 258:
					return "BitsPerSample";
				case 259:
					return "Compression";
				case 262:
					return "PhotometricInterp";
				case 263:
					return "ThreshHolding";
				case 264:
					return "CellWidth";
				case 265:
					return "CellHeight";
				case 266:
					return "FillOrder";
				case 269:
					return "DocumentName";
				case 270:
					return "ImageDescription";
				case 271:
					return "EquipMake";
				case 272:
					return "EquipModel";
				case 273:
					return "StripOffsets";
				case 274:
					return "Orientation";
				case 277:
					return "SamplesPerPixel";
				case 278:
					return "RowsPerStrip";
				case 279:
					return "StripBytesCount";
				case 280:
					return "MinSampleValue";
				case 281:
					return "MaxSampleValue";
				case 282:
					return "XResolution";
				case 283:
					return "YResolution";
				case 284:
					return "PlanarConfig";
				case 285:
					return "PageName";
				case 286:
					return "XPosition";
				case 287:
					return "YPosition";
				case 288:
					return "FreeOffset";
				case 289:
					return "FreeByteCounts";
				case 290:
					return "GrayResponseUnit";
				case 291:
					return "GrayResponseCurve";
				case 292:
					return "T4Option";
				case 293:
					return "T6Option";
				case 296:
					return "ResolutionUnit";
				case 297:
					return "PageNumber";
				case 301:
					return "TransferFunction";
				case 305:
					return "SoftwareUsed";
				case 306:
					return "DateTime";
				case 315:
					return "Artist";
				case 316:
					return "HostComputer";
				case 317:
					return "Predictor";
				case 318:
					return "WhitePoint";
				case 319:
					return "PrimaryChromaticities";
				case 320:
					return "ColorMap";
				case 321:
					return "HalftoneHints";
				case 322:
					return "TileWidth";
				case 323:
					return "TileLength";
				case 324:
					return "TileOffset";
				case 325:
					return "TileByteCounts";
				case 332:
					return "InkSet";
				case 333:
					return "InkNames";
				case 334:
					return "NumberOfInks";
				case 336:
					return "DotRange";
				case 337:
					return "TargetPrinter";
				case 338:
					return "ExtraSamples";
				case 339:
					return "SampleFormat";
				case 340:
					return "SMinSampleValue";
				case 341:
					return "SMaxSampleValue";
				case 342:
					return "TransferRange";
				case 512:
					return "Proc";
				case 513:
					return "InterFormat";
				case 514:
					return "InterLength";
				case 515:
					return "RestartInterval";
				case 517:
					return "LosslessPredictors";
				case 518:
					return "PointTransforms";
				case 519:
					return "QTables";
				case 520:
					return "DCTables";
				case 521:
					return "ACTables";
				case 529:
					return "YCbCrCoefficients";
				case 530:
					return "YCbCrSubsampling";
				case 531:
					return "YCbCrPositioning";
				case 532:
					return "REFBlackWhite";
				case 769:
					return "Gamma";
				case 770:
					return "ICCProfileDescriptor";
				case 800:
					return "ImageTitle";
				case 20481:
					return "ResolutionXUnit";
				case 20482:
					return "ResolutionYUnit";
				case 20483:
					return "ResolutionXLengthUnit";
				case 20484:
					return "ResolutionYLengthUnit";
				case 20496:
					return "JPEGQuality";
				case 33432:
					return "Copyright";
				case 33434:
					return "ExposureTime";
				case 33437:
					return "FNumber";
				case 34665:
					return "IFD";
				case 34675:
					return "ICCProfile";
				case 34850:
					return "ExposureProg";
				case 34852:
					return "SpectralSense";
				case 34853:
					return "GpsIFD";
				case 34855:
					return "ISOSpeed";
				case 34856:
					return "OECF";
				case 36864:
					return "ExifVersion";
				case 36867:
					return "DateTimeOriginal";
				case 36868:
					return "DateTimeDigitized";
				case 37121:
					return "ComponentsConfiguration";
				case 37122:
					return "CompBPP";
				case 37377:
					return "ShutterSpeed";
				case 37378:
					return "Aperture";
				case 37379:
					return "Brightness";
				case 37380:
					return "ExposureBias";
				case 37381:
					return "MaxAperture";
				case 37382:
					return "SubjectDist";
				case 37383:
					return "MeteringMode";
				case 37384:
					return "LightSource";
				case 37385:
					return "Flash";
				case 37386:
					return "FocalLength";
				case 37396:
					return "SubjectArea";
				case 37500:
					return "MakerNote";
				case 37510:
					return "UserComment";
				case 37520:
					return "DTSubsec";
				case 37521:
					return "DTOrigSS";
				case 37522:
					return "DTDigSS";
				case 40960:
					return "FlashpixVersion";
				case 40961:
					return "ColorSpace";
				case 40962:
					return "PixXDimension";
				case 40963:
					return "PixYDimension";
				case 40964:
					return "RelatedWav";
				case 40965:
					return "Interop";
				case 41483:
					return "FlashEnergy";
				case 41484:
					return "SpatialFR";
				case 41486:
					return "FocalXRes";
				case 41487:
					return "FocalYRes";
				case 41488:
					return "FocalResUnit";
				case 41492:
					return "SubjectLoc";
				case 41493:
					return "ExposureIndex";
				case 41495:
					return "SensingMethod";
				case 41728:
					return "FileSource";
				case 41729:
					return "SceneType";
				case 41730:
					return "CfaPattern";
				case 41985:
					return "CustomRendered";
				case 41986:
					return "ExposureMode";
				case 41987:
					return "WhiteBalance";
				case 41988:
					return "DigitalZoomRatio";
				case 41989:
					return "FocalLengthIn35mmFilm";
				case 41990:
					return "SceneCaptureType";
				case 41991:
					return "GainControl";
				case 41992:
					return "Contrast";
				case 41993:
					return "Saturation";
				case 41994:
					return "Sharpness";
				case 41995:
					return "DeviceSettingDescription";
				case 41996:
					return "SubjectDistanceRange";
				case 42016:
					return "ImageUniqueID";
			}
			return "";
		}

		public static string GetPropertyDescriptionFromId(int nID)
		{
			switch(nID)
			{
				case 256:
					return "Number of pixels per row";
				case 257:
					return "Number of pixel rows";
				case 258:
					return "Number of bits per color component";
				case 259:
					return "Compression scheme used for the image data";
				case 262:
					return "How pixel data will be interpreted";
				case 263:
					return "Technique used to convert from gray pixels to black and white pixels";
				case 264:
					return "Width of the dithering or halftoning matrix";
				case 265:
					return "Height of the dithering or halftoning matrix";
				case 266:
					return "Logical order of bits in a byte";
				case 269:
					return "Null-terminated character string that specifies the name of the document from which the image was scanned";
				case 270:
					return "Null-terminated character string that specifies the title of the image";
				case 271:
					return "Null-terminated character string that specifies the manufacturer of the equipment used to record the image";
				case 272:
					return "Null-terminated character string that specifies the model name or model number of the equipment used to record the image";
				case 273:
					return "For each strip, the byte offset of that strip";
				case 274:
					return "For each strip, the byte offset of that strip:\n"+
						   "1 - The 0th row is at the visual top of the image, and the 0th column is the visual left-hand side\n"+
						   "2 - The 0th row is at the visual top of the image, and the 0th column is the visual right-hand side\n"+
						   "3 - The 0th row is at the visual bottom of the image, and the 0th column is the visual right-hand side\n"+
						   "4 - The 0th row is at the visual bottom of the image, and the 0th column is the visual left-hand side\n"+
						   "5 - The 0th row is the visual left-hand side of of the image, and the 0th column is the visual top\n"+
						   "6 - The 0th row is the visual right-hand side of of the image, and the 0th column is the visual top\n"+
						   "7 - The 0th row is the visual right-hand side of of the image, and the 0th column is the visual bottom\n"+
						   "8 - The 0th row is the visual left-hand side of of the image, and the 0th column is the visual bottom\n";
				case 277:
					return "Number of color components per pixel";
				case 278:
					return "Number of rows per strip";
				case 279:
					return "For each strip, the total number of bytes in that strip";
				case 280:
					return "For each color component, the minimum value assigned to that component";
				case 281:
					return "For each color component, the maximum value assigned to that component";
				case 282:
					return "Number of pixels per unit in the image width (x) direction. The unit is specified by ResolutionUnit";
				case 283:
					return "Number of pixels per unit in the image height (y) direction. The unit is specified by ResolutionUnit";
				case 284:
					return "Whether pixel components are recorded in chunky or planar format";
				case 285:
					return "Null-terminated character string that specifies the name of the page from which the image was scanned";
				case 286:
					return "Offset from the left side of the page to the left side of the image. The unit of measure is specified by ResolutionUnit";
				case 287:
					return "Offset from the top of the page to the top of the image. The unit of measure is specified by ResolutionUnit";
				case 288:
					return "For each string of contiguous unused bytes, the byte offset of that string";
				case 289:
					return "For each string of contiguous unused bytes, the number of bytes in that string";
				case 290:
					return "Precision of the number specified by GrayResponseCurve. 1 specifies tenths, 2 specifies hundredths, 3 specifies thousandths, and so on";
				case 291:
					return "For each possible pixel value in a grayscale image, the optical density of that pixel value";
				case 292:
					return "Set of flags that relate to T4 encoding";
				case 293:
					return "Set of flags that relate to T6 encoding";
				case 296:
					return "Unit of measure for the horizontal resolution and the vertical resolution:\n"+
						   "1 - No absolute unit of measurement. Used for images that may have a non-square aspect ratio, but no meaningful absolute dimensions\n"+
						   "2 - Inches\n"+
						   "3 - Centimeters";
				case 297:
					return "Page number of the page from which the image was scanned";
				case 301:
					return "Tables that specify transfer functions for the image";
				case 305:
					return "Null-terminated character string that specifies the name and version of the software or firmware of the device used to generate the image";
				case 306:
					return "Date and time the image was created";
				case 315:
					return "Null-terminated character string that specifies the name of the person who created the image";
				case 316:
					return "Null-terminated character string that specifies the computer and/or operating system used to create the image";
				case 317:
					return "Type of prediction scheme that was applied to the image data before the encoding scheme was applied";
				case 318:
					return "Chromaticity of the white point of the image";
				case 319:
					return "For each of the three primary colors in the image, the chromaticity of that color";
				case 320:
					return "Color palette (lookup table) for a palette-indexed image";
				case 321:
					return "Information used by the halftone function";
				case 322:
					return "Number of pixel columns in each tile";
				case 323:
					return "Number of pixel rows in each tile";
				case 324:
					return "For each tile, the byte offset of that tile";
				case 325:
					return "For each tile, the number of bytes in that tile";
				case 332:
					return "Set of inks used in a separated image";
				case 333:
					return "Sequence of concatenated, null-terminated, character strings that specify the names of the inks used in a separated image";
				case 334:
					return "Number of inks";
				case 336:
					return "Color component values that correspond to a 0 percent dot and a 100 percent dot";
				case 337:
					return "Null-terminated character string that describes the intended printing environment";
				case 338:
					return "Number of extra color components. For example, one extra component might hold an alpha value";
				case 339:
					return "For each color component, the numerical format (unsigned, signed, floating point) of that component";
				case 340:
					return "For each color component, the minimum value of that component";
				case 341:
					return "For each color component, the maximum value of that component";
				case 342:
					return "Table of values that extends the range of the transfer function";
				case 512:
					return "JPEG compression process";
				case 513:
					return "Offset to the start of a JPEG bitstream";
				case 514:
					return "Length, in bytes, of the JPEG bitstream";
				case 515:
					return "Length of the restart interval";
				case 517:
					return "For each color component, a lossless predictor-selection value for that component";
				case 518:
					return "For each color component, a point transformation value for that component";
				case 519:
					return "For each color component, the offset to the quantization table for that component";
				case 520:
					return "For each color component, the offset to the DC Huffman table (or lossless Huffman table) for that component";
				case 521:
					return "For each color component, the offset to the AC Huffman table for that component";
				case 529:
					return "Coefficients for transformation from RGB to YCbCr image data";
				case 530:
					return "Sampling ratio of chrominance components in relation to the luminance component";
				case 531:
					return "Position of chrominance components in relation to the luminance component";
				case 532:
					return "Reference black point value and reference white point value";
				case 769:
					return "Gamma value attached to the image. The gamma value is stored as a rational number (pair of long) with a numerator of 100000. For example, a gamma value of 2.2 is stored as the pair (100000, 45455)";
				case 770:
					return "Null-terminated character string that identifies an ICC profile";
				case 800:
					return "Null-terminated character string that specifies the title of the image";
				case 20481:
					return "Units in which to display horizontal resolution:\n"+
						   "1 - Pixels per inch\n"+
						   "2 - Pixels per centimeter";
				case 20482:
					return "Units in which to display vertical resolution:\n"+
						   "1 - Pixels per inch\n"+
						   "2 - Pixels per centimeter";
				case 20483:
					return "Units in which to display the image width:\n"+
						   "1 - Inches\n"+
						   "2 - Centimeters\n"+
						   "3 - Points\n"+
						   "4 - Picas\n"+
						   "5 - Columns";
				case 20484:
					return "Units in which to display the image height:\n"+
						   "1 - Inches\n"+
						   "2 - Centimeters\n"+
						   "3 - Points\n"+
						   "4 - Picas\n"+
						   "5 - Columns";
				case 20496:
					return "Private tag used by the Adobe Photoshop format";
				case 33432:
					return "Null-terminated character string that contains copyright information";
				case 33434:
					return "Exposure time, measured in seconds";
				case 33437:
					return "F Number";
				case 34665:
					return "Private tag used by GDI+. GDI+ uses this tag to locate Exif-specific information";
				case 34675:
					return "ICC profile embedded in the image";
				case 34850:
					return "Class of the program used by the camera to set exposure when the picture is taken:\n"+
						   "0 - Not defined\n"+
						   "1 - Manual\n"+ 
						   "2 - Normal program\n"+
						   "3 - Aperture priority\n"+
						   "4 - Shutter priority\n"+
						   "5 - Creative program (biased toward depth of field)\n"+
						   "6 - Action program (biased toward fast shutter speed)\n"+
						   "7 - Portrait mode (for closeup photos with the background out of focus)\n"+
						   "8 - Landscape mode (for landscape photos with the background in focus)";
				case 34852:
					return "Null-terminated character string that specifies the spectral sensitivity of each channel of the camera used. The string is compatible with the standard developed by the ASTM Technical Committee";
				case 34853:
					return "Offset to a block of GPS property items. Property items whose tags have the prefix Gps are stored in the GPS block. The GPS property items are defined in the EXIF specification. GDI+ uses this tag to locate GPS information, but GDI+ does not expose this tag for public use";
				case 34855:
					return "ISO speed and ISO latitude of the camera or input device as specified in ISO 12232";
				case 34856:
					return "Optoelectronic conversion function (OECF) specified in ISO 14524. The OECF is the relationship between the camera optical input and the image values";
				case 36864:
					return "Version of the EXIF standard supported. Nonexistence of this field is taken to mean nonconformance to the standard. Conformance to the standard is indicated by recording 0210 as a 4-byte ASCII string. Because the type is Undefined, there is no NULL terminator";
				case 36867:
					return "Date and time when the original image data was generated. For a DSC, the date and time when the picture was taken. The format is YYYY:MM:DD HH:MM:SS with time shown in 24-hour format and the date and time separated by one blank character (0x2000). The character string length is 20 bytes including the NULL terminator. When the field is empty, it is treated as unknown";
				case 36868:
					return "Date and time when the image was stored as digital data. If, for example, an image was captured by DSC and at the same time the file was recorded, then DateTimeOriginal and DateTimeDigitized will have the same contents. The format is YYYY:MM:DD HH:MM:SS with time shown in 24-hour format and the date and time separated by one blank character (0x2000). The character string length is 20 bytes including the NULL terminator. When the field is empty, it is treated as unknown";
				case 37121:
					return "Information specific to compressed data. The channels of each component are arranged in order from the first component to the fourth. For uncompressed data, the data arrangement is given in the PhotometricInterp tag. However, because PhotometricInterp can only express the order of Y, Cb, and Cr, this tag is provided for cases when compressed data uses components other than Y, Cb, and Cr and to support other sequences";
				case 37122:
					return "Information specific to compressed data. The compression mode used for a compressed image is indicated in unit BPP";
				case 37377:
					return "Shutter speed. The unit is the Additive System of Photographic Exposure (APEX) value";
				case 37378:
					return "Lens aperture. The unit is the Additive System of Photographic Exposure (APEX) value";
				case 37379:
					return "Brightness value. The unit is the Additive System of Photographic Exposure (APEX) value. Ordinarily it is given in the range of -99.99 to 99.99";
				case 37380:
					return "Exposure bias. The unit is the Additive System of Photographic Exposure (APEX) value. Ordinarily it is given in the range -99.99 to 99.99";
				case 37381:
					return "Smallest F number of the lens. The unit is the Additive System of Photographic Exposure (APEX) value. Ordinarily it is given in the range of 00.00 to 99.99, but it is not limited to this range";
				case 37382:
					return "Distance to the subject, measured in meters";
				case 37383:
					return "Metering Mode:\n"+
						   "0 - Unknown\n"+
						   "1 - Average\n"+
						   "2 - CenterWeightedAverage\n"+
						   "3 - Spot\n"+
						   "4 - MultiSpot\n"+
						   "5 - Pattern\n"+
						   "6 - Partial";
				case 37384:
					return "Type of light source:\n"+
						   "0 - Unknown\n"+
						   "1 - Daylight\n"+
						   "2 - Fluorescent\n"+
						   "3 - Tungsten\n"+
						   "17 - Standard light A\n"+
						   "18 - Standard light B\n"+
						   "19 - Standard light C\n"+
						   "20 - D55\n"+
						   "21 - D65\n"+
						   "22 - D75";

				case 37385:
					return "Flash status. This tag is recorded when an image is taken using a strobe light (flash). Bit 0 indicates the flash firing status, and bits 1 and 2 indicate the flash return status:\n"+
						   "0 - Flash did not fire\n"+
						   "1 - Flash fired\n"+
						   "5 - Strobe return light not detected\n"+
						   "7 - Strobe return light detected";
				case 37386:
					return "Actual focal length, in millimeters, of the lens. Conversion is not made to the focal length of a 35 millimeter film camera";
				case 37396:
					return "Indicates the location and area of the main subject in the overall scene";
				case 37500:
					return "Note tag. A tag used by manufacturers of EXIF writers to record information. The contents are up to the manufacturer";
				case 37510:
					return "Comment tag. A tag used by EXIF users to write keywords or comments about the image besides those in ImageDescription and without the character-code limitations of the ImageDescription tag";
				case 37520:
					return "Null-terminated character string that specifies a fraction of a second for the DateTime tag";
				case 37521:
					return "Null-terminated character string that specifies a fraction of a second for the ExifDTOrig tag";
				case 37522:
					return "Null-terminated character string that specifies a fraction of a second for the ExifDTDigitized tag";
				case 40960:
					return "FlashPix format version supported by an FPXR file. If the FPXR function supports FlashPix format version 1.0, this is indicated similarly to ExifVer by recording 0100 as a 4-byte ASCII string. Because the type is Undefined, there is no NULL terminator";
				case 40961:
					return "Color space specifier. Normally sRGB (=1) is used to define the color space based on the PC monitor conditions and environment. If a color space other than sRGB is used, Uncalibrated (=0xFFFF) is set. Image data recorded as Uncalibrated can be treated as sRGB when it is converted to FlashPix";
				case 40962:
					return "Information specific to compressed data. When a compressed file is recorded, the valid width of the meaningful image must be recorded in this tag, whether or not there is padding data or a restart marker. This tag should not exist in an uncompressed file";
				case 40963:
					return "Information specific to compressed data. When a compressed file is recorded, the valid height of the meaningful image must be recorded in this tag whether or not there is padding data or a restart marker. This tag should not exist in an uncompressed file. Because data padding is unnecessary in the vertical direction, the number of lines recorded in this valid image height tag will be the same as that recorded in the SOF";
				case 40964:
					return "The name of an audio file related to the image data. The only relational information recorded is the EXIF audio file name and extension (an ASCII string that consists of 8 characters plus a period (.), plus 3 characters). The path is not recorded. When you use this tag, audio files must be recorded in conformance with the EXIF audio format. Writers can also store audio data within APP2 as FlashPix extension stream data";
				case 40965:
					return "Offset to a block of property items that contain interoperability information";
				case 41483:
					return "Strobe energy, in Beam Candle Power Seconds (BCPS), at the time the image was captured";
				case 41484:
					return "Camera or input device spatial frequency table and SFR values in the image width, image height, and diagonal direction, as specified in ISO 12233";
				case 41486:
					return "Number of pixels in the image width (x) direction per unit on the camera focal plane. The unit is specified in ExifFocalResUnit";
				case 41487:
					return "Number of pixels in the image height (y) direction per unit on the camera focal plane. The unit is specified in ExifFocalResUnit";
				case 41488:
					return "Unit of measure for ExifFocalXRes and ExifFocalYRes:\n"+
						   "1 - No absolute unit of measurement\n"+
						   "2 - Inch\n"+
						   "3 - Centimeter";
				case 41492:
					return "Location of the main subject in the scene. The value of this tag represents the pixel at the center of the main subject relative to the left edge. The first value indicates the column number, and the second value indicates the row number";
				case 41493:
					return "Exposure index selected on the camera or input device at the time the image was captured";
				case 41495:
					return "Image sensor type on the camera or input device:\n"+
						   "1 - Not defined\n"+
						   "2 - One-chip color area sensor\n"+
						   "3 - Two-chip color area sensor\n"+
						   "4 - Three-chip color area sensor\n"+
						   "5 - Color sequential area sensor\n"+
						   "7 - Trilinear sensor\n"+
						   "8 - Color sequential linear sensor\n";
				case 41728:
					return "The image source. If a DSC recorded the image, the value of this tag is 3";
				case 41729:
					return "The type of scene. If a DSC recorded the image, the value of this tag must be set to 1, indicating that the image was directly photographed";
				case 41730:
					return "The color filter array (CFA) geometric pattern of the image sensor when a one-chip color area sensor is used. It does not apply to all sensing methods";
				case 41985:
					return "Indicates the use of special processing on image data, such as rendering geared to output:\n"+
						   "0 - Normal process\n"+
						   "1 - Custom process";
				case 41986:
					return "Indicates the exposure mode set when the image was shot:\n"+
						   "0 - Auto exposure\n"+
						   "1 - Manual exposure\n"+
						   "2 - Auto bracket";
				case 41987:
					return "Indicates the white balance mode set when the image was shot:\n"+
						   "0 - Auto white balance\n"+
						   "1 - Manual white balance";
				case 41988:
					return "Indicates the digital zoom ratio when the image was shot. If the numerator of the recorded value is 0, this indicates that digital zoom was not used";
				case 41989:
					return "Indicates the equivalent focal length assuming a 35mm film camera, in mm";
				case 41990:
					return "Indicates the type of scene that was shot:\n"+
						   "0 - Standard\n"+
						   "1 - Landscape\n"+
						   "2 - Portrait\n"+
						   "3 - Night scene";
				case 41991:
					return "Indicates the degree of overall image gain adjustment:\n"+
						   "0 - None\n"+
						   "1 - Low gain up\n"+
						   "2 - High gain up\n"+
						   "3 - Low gain down\n"+
						   "4 - High gain down";
				case 41992:
					return "Indicates the direction of contrast processing applied by the camera when the image was shot:\n"+
						   "0 - Normal\n"+
						   "1 - Soft\n"+
						   "2 - Hard";
				case 41993:
					return "Indicates the direction of saturation processing applied by the camera when the image was shot:\n"+
						   "0 - Normal\n"+
						   "1 - Low saturation\n"+
						   "2 - High saturation";
				case 41994:
					return "Indicates the direction of sharpness processing applied by the camera when the image was shot:\n"+
						   "0 - Normal\n"+
						   "1 - Low sharpness\n"+
						   "2 - High sharpness";
				case 41995:
					return "This tag indicates information on the picture-taking conditions of a particular camera model";
				case 41996:
					return "Indicates the distance to the subject:\n"+
							"0 - Unknown\n"+
							"1 - Macro\n"+
							"2 - Close view\n"+
							"3 - Distant view";
				case 42016:
					return "Indicates an identifier assigned uniquely to each image";
			}
			return "";
		}

		/// <summary>
		/// Formats a PropertyItem's byte data.</summary>
		/// <returns>A string representation of the data.</returns>
		public static string GetPropertyValue(PropertyItem propItem)
		{
			if (propItem == null)
				return String.Empty;

			string strRet = "";

			switch(propItem.Type) 
			{
				case PropertyTagTypeByte:
					strRet = FormatTagByte(propItem);
					break;
				case PropertyTagTypeASCII:
					strRet = FormatTagAscii(propItem);
					break;
				case PropertyTagTypeShort:
					strRet = FormatTagShort(propItem);
					break;
				case PropertyTagTypeLong:
					strRet = FormatTagLong(propItem);
					break;
				case PropertyTagTypeRational:
					strRet = FormatTagRational(propItem);
					break;
				case PropertyTagTypeUndefined:
					strRet = FormatTagUndefined(propItem);
					break;
				case PropertyTagTypeSLong:
					strRet = FormatTagSLong(propItem);
					break;
				case PropertyTagTypeSRational:
					strRet = FormatTagSRational(propItem);
					break;
				default:
					strRet = "";
					break;
			}
			return strRet;
		}
		
		/// <summary>Format a Byte tag.</summary>
		private static string FormatTagByte(PropertyItem propItem) 
		{
			return BitConverter.ToString(propItem.Value, 0, propItem.Len);
		}

		/// <summary>Format an ASCII tag.</summary>
		private static string FormatTagAscii(PropertyItem propItem) 
		{
			string strRet;
			System.Text.ASCIIEncoding encoding = new System.Text.ASCIIEncoding();
			strRet = encoding.GetString(propItem.Value, 0, propItem.Len - 1);

			return strRet;
		}

		/// <summary>Format a Short tag (unsigned).</summary>
		private static string FormatTagShort(PropertyItem propItem) 
		{
			string strRet = "";
			for (int i = 0; i < propItem.Len; i = i + BYTEJUMP_SHORT) 
			{
				System.UInt16 val = BitConverter.ToUInt16(propItem.Value, i);
				strRet += val.ToString();
				if (i + BYTEJUMP_SHORT < propItem.Len)
					strRet += " ";
			}
			return strRet;
		}

		/// <summary>Format a Long tag (unsigned).</summary>
		private static string FormatTagLong(PropertyItem propItem) 
		{
			string strRet = "";
			for (int i = 0; i < propItem.Len; i = i + BYTEJUMP_LONG) 
			{
				System.UInt32 val = BitConverter.ToUInt32(propItem.Value, i);
				strRet += val.ToString();
				if (i + BYTEJUMP_LONG < propItem.Len)
					strRet += " ";
			}
			return strRet;
		}

		/// <summary>Format a Rational tag (unsigned).</summary>
		private static string FormatTagRational(PropertyItem propItem) 
		{
			string strRet = "";
			for (int i = 0; i < propItem.Len; i = i + BYTEJUMP_RATIONAL) 
			{
				System.UInt32 numer = BitConverter.ToUInt32(propItem.Value, i);
				System.UInt32 denom = BitConverter.ToUInt32(propItem.Value, i + BYTEJUMP_LONG);
				double dbl;
				if (denom  == 0)
					dbl = 0.0;
				else
					dbl = (double)numer / (double)denom;
				strRet += dbl.ToString(DOUBLETYPE_FORMAT);
				
				if (i + BYTEJUMP_RATIONAL < propItem.Len)
					strRet += " ";
			}
			return strRet;
		}

		/// <summary>Format a Undefined tag.</summary>
		private static string FormatTagUndefined(PropertyItem propItem) 
		{
			string strRet;
			strRet = BitConverter.ToString(propItem.Value, 0, propItem.Len);
			return strRet;
		}

		/// <summary>Format a SLong tag (signed).</summary>
		private static string FormatTagSLong(PropertyItem propItem) 
		{
			string strRet = "";
			for (int i = 0; i < propItem.Len; i = i + BYTEJUMP_SLONG) 
			{
				System.Int32 val = BitConverter.ToInt32(propItem.Value, i);
				strRet += val.ToString();
				if (i + BYTEJUMP_SLONG < propItem.Len)
					strRet += " ";
			}
			return strRet;
		}

		/// <summary>Format a SRational tag (signed).</summary>
		private static string FormatTagSRational(PropertyItem propItem) 
		{
			string strRet = "";
			for (int i = 0; i < propItem.Len; i = i + BYTEJUMP_SRATIONAL) 
			{
				System.Int32 numer = BitConverter.ToInt32(propItem.Value, i);
				System.Int32 denom = BitConverter.ToInt32(propItem.Value, i + BYTEJUMP_SLONG);
				double dbl;
				if (denom  == 0)
					dbl = 0.0;
				else
					dbl = (double)numer / (double)denom;
				strRet += dbl.ToString(DOUBLETYPE_FORMAT);
				
				if (i + BYTEJUMP_SRATIONAL < propItem.Len)
					strRet += " ";
			}
			return strRet;
		}
	}
}
